@@ -1,5 +1,5 @@
 package Yancy::Model::Schema;
-our $VERSION = '1.083';
+our $VERSION = '1.084';
 # ABSTRACT: Interface to a single schema
 
 #pod =head1 SYNOPSIS
@@ -50,13 +50,13 @@ has model => sub { die 'model is required' };
 
 has name => sub { die 'name is required' };
 
-#pod =attr info
+#pod =attr json_schema
 #pod
 #pod The JSON Schema for this schema.
 #pod
 #pod =cut
 
-has info => sub { die 'info is required' };
+has json_schema => sub { die 'json_schema is required' };
 
 sub _backend { shift->model->backend };
 has _item_class => sub {
@@ -68,7 +68,7 @@ sub _log { shift->model->log };
 sub new {
   my ( $class, @args ) = @_;
   my $self = $class->SUPER::new( @args );
-  $self->_check_info;
+  $self->_check_json_schema;
   return $self;
 }
 
@@ -81,7 +81,7 @@ sub new {
 
 sub id_field {
     my ( $self ) = @_;
-    return $self->info->{'x-id-field'} // 'id';
+    return $self->json_schema->{'x-id-field'} // 'id';
 }
 
 #pod =method build_item
@@ -104,7 +104,7 @@ sub build_item {
 
 sub validate {
     my ( $self, $item, %opt ) = @_;
-    my $schema = $self->info;
+    my $schema = $self->json_schema;
 
     if ( $opt{ properties } ) {
         # Only validate these properties
@@ -271,22 +271,26 @@ sub delete {
     return $self->_backend->delete( $self->name, $id );
 }
 
-sub _check_info {
+sub _check_json_schema {
     my ( $self ) = @_;
     my $name = $self->name;
-    my $info = $self->info;
+    my $json_schema = $self->json_schema;
 
     # Deprecate x-view. Yancy::Model is a much better
     # solution to that.
     derp q{x-view is deprecated and will be removed in v2. }
         . q{Use Yancy::Model or your database's CREATE VIEW instead}
-        if $info->{'x-view'};
+        if $json_schema->{'x-view'};
 
-    $info->{ type } //= 'object';
-    my $real_name = ( $info->{'x-view'} || {} )->{schema} // $name;
-    my $props = $info->{properties}
-        || $self->model->schema( $real_name )->info->{properties};
-    my $id_field = $info->{ 'x-id-field' } // 'id';
+    $json_schema->{ type } //= 'object';
+    my $props = $json_schema->{properties};
+    if ( $json_schema->{'x-view'} && !$props ) {
+        my $real_name = $json_schema->{'x-view'}->{schema};
+        my $real_schema = $self->model->schema( $real_name )
+          // die qq{Could not find x-view schema "$real_name" for schema "$name"};
+        $props = $real_schema->json_schema->{properties};
+    }
+    my $id_field = $self->id_field;
     # ; say "$name ($real_name) ID field: $id_field";
     # ; use Data::Dumper;
     # ; say Dumper $props;
@@ -314,7 +318,7 @@ Yancy::Model::Schema - Interface to a single schema
 
 =head1 VERSION
 
-version 1.083
+version 1.084
 
 =head1 SYNOPSIS
 
@@ -348,7 +352,7 @@ The L<Yancy::Model> object that created this schema object.
 
 The name of the schema.
 
-=head2 info
+=head2 json_schema
 
 The JSON Schema for this schema.
 
